'use strict';
const fs = require('fs');
const semver = require('semver');
const execa = require('execa');
const pify = require('pify');
const Listr = require('listr');
const np = require('np');

const fsP = pify(fs);

function getCurrentVersion(_currentVersion) {
	if (semver.valid(_currentVersion)) {
		return Promise.resolve(_currentVersion);
	}
	return fsP.readFile('package.json', 'utf8')
		.then(JSON.parse)
		.then(pkgJson => {
			return pkgJson.version;
		});
}

module.exports = (_currentVersion, _options) => {
	const options = _options || {};
	const _np = options.np || np;
	const _getGitBranch = options.getGitBranch || (() => execa.stdout('git', ['symbolic-ref', '--short', 'HEAD']));

	let currentVersion;
	let currentVersionIsPrerelease = false;
	let nextVersion = '';
	let nextVersionIsPrerelease = false;
	let tag = null;

	const tasks = new Listr([{
		title: 'Generate new version number using semver and current branch',
		task: () => new Listr([
			{
				title: 'Check branch',
				task: () => {
					return _getGitBranch().then(branch => {
						if (branch !== 'master') {
							nextVersionIsPrerelease = true;
							tag = branch;
						}
					});
				}
			},
			{
				title: 'Run Semver',
				task: () => {
					if (nextVersionIsPrerelease && currentVersionIsPrerelease) {
						nextVersion = semver.inc(currentVersion, 'prerelease');
					} else if (nextVersionIsPrerelease) {
						nextVersion = semver.inc(currentVersion, 'prepatch', tag);
					} else {
						nextVersion = semver.inc(currentVersion, 'patch');
					}
					// console.log(`nextVersion: ${nextVersion}`)

					return Promise.resolve();
				}
			}
		])
	}]);

	return getCurrentVersion(_currentVersion)
		.then(_currentVersion => {
			currentVersion = _currentVersion;
			currentVersionIsPrerelease = Boolean(semver.prerelease(currentVersion));
		})
		.then(() => tasks.run())
		.then(() => {
			const opts = {
				anyBranch: true
			};
			if (nextVersionIsPrerelease) {
				opts.tag = tag;
			}
			// console.log(`np(${nextVersion}, ${JSON.stringify(opts)})`);
			return _np(nextVersion, opts);
		})
		.then(() => fsP.readFile('package.json', 'utf8'))
		.then(JSON.parse);
};
